% simulation study of identified set procedure

% clear all old data
clear;

% read in data
% variables are: sex, hrs/week worked, salary1, salary2
cpsdat = csvread('cps00_mf.csv');

% make the support of the regressors
supp = zeros(2,2);
for i = 1:2
    supp(i,:) = [1 i-1];
end

% define some variables
M = 100;						% number of simulations
n = 1000						% number of observations to draw
B = 200;						% number of subsamples to use
ub = 10^6;						% assumed upper bound on salary1 + salary2
ncps = size(cpsdat,1);			% cardinality of support of true distribution
data = zeros(ncps,4);			% the support of the true distribution
taup = zeros(2,2);				% tau's for population
np = zeros(2,1);				% room for nj's for population
crit = zeros(B,1);				% the estimated critical values
thresh = zeros(25,1);			% room to save thresholds
bvec = 15:5:85;					% vector of subsample sizes to try
numb = size(bvec,2);			% number of different subsamples

% zero out different count variables
percoverss = zeros(numb,1);
percoversd = zeros(numb,1);
perimprss = zeros(numb,1);
avgiter = zeros(numb,1);
periter2 = zeros(numb,1);

% room for all the thresholds
ssthresh = zeros(M,numb);
sdthresh = zeros(M,numb);
sd2thresh = zeros(M,numb);
objmax = zeros(M,numb);

% figure out the censoring level
wages = zeros(ncps,1);
swages = zeros(ncps,1);
wages = cpsdat(:,3) + cpsdat(:,4);
swages = sort(wages);
lb = swages(ceil(.95*ncps),1);

% make support of the true distribution
for i = 1:ncps
	if (wages(i,1) > lb)
	   data(i,1) = lb;
	   data(i,2) = ub;
	else
	   data(i,1) = wages(i,1);
	   data(i,2) = wages(i,1);
	end
end
data(:,1) = log(data(:,1)./(52*cpsdat(:,2)));
data(:,2) = log(data(:,2)./(52*cpsdat(:,2)));
data(:,3) = ones(ncps,1);
data(:,4) = cpsdat(:,1);

% make taup and np for the population
ind = zeros(1,ncps);
for i = 1:2
    ind = min((kron(supp(i,:),ones(ncps,1)) == data(:,3:4))');
    np(i,1) = sum(ind);
    if (np(i,1) > 0)   
       taup(i,1) = ind*data(:,1)/np(i,1);
       taup(i,2) = ind*data(:,2)/np(i,1);
    else
       taup(i,1) = 0;
       taup(i,2) = ub;
    end
end

% compute four extreme points of identified set
theta = zeros(2,4);
theta(:,1) = [taup(1,1) ; taup(2,1) - taup(1,1)];
theta(:,2) = [taup(1,1) ; taup(2,2) - taup(1,1)];
theta(:,3) = [taup(1,2) ; taup(2,1) - taup(1,2)];
theta(:,4) = [taup(1,2) ; taup(2,2) - taup(1,2)];

% loop over different subsample sizes
for c = 1:numb

% the subsample size we are on
b = bvec(c);

simdat = zeros(n,4);			% the simulated data
subsimdat = zeros(b,4);			% room for the subsampled data
taun = zeros(2,2);				% tau's for entire sample
taub = zeros(2,2,B);			% tau's for each subsample
nb = zeros(2,B);				% room for nj's for each subsample
nn = zeros(2,1);				% room for nj's for entire sample

% loop over the different simulations
for m = 1:M

% the simulation we are on
b
m

% draw a random sample of length n from distribution 
% uniformly distributed over elements of data

	% generate n random draws from 1:ncps
	ind = zeros(n,1);
	ind = ceil(ncps.*rand(n,1));

	% set the new data
	for i = 1:n
		simdat(i,:) = data(ind(i),:);
	end

% draw subsamples, make taub and nb
ind = zeros(1,b);
for i = 1:B
    samp = randperm(n);
    for j = 1:b
        subsimdat(j,:) = simdat(samp(j),:);
    end
    for j = 1:2
        ind = min((kron(supp(j,:),ones(b,1)) == subsimdat(:,3:4))');
        nb(j,i) = sum(ind);
        if (nb(j,i) > 0)
           taub(j,1,i) = ind*subsimdat(:,1)/nb(j,i);
           taub(j,2,i) = ind*subsimdat(:,2)/nb(j,i);
        else
           taub(j,1,i) = 0;
           taub(j,2,i) = ub;
        end
    end
end

% make taun and nn for the entire sample
ind = zeros(1,n);
for i = 1:2
    ind = min((kron(supp(i,:),ones(n,1)) == simdat(:,3:4))');
    nn(i,1) = sum(ind);
    if (nn(i,1) > 0)   
       taun(i,1) = ind*simdat(:,1)/nn(i,1);
       taun(i,2) = ind*simdat(:,2)/nn(i,1);
    else
       taun(i,1) = 0;
       taun(i,2) = ub;
    end
end

% compute critical value from stepdown procedure
count = 0;
flag = 0;
eps = 1000;
thresh = zeros(25,1);
while (flag == 0)
    % just a counter
    count = count + 1;
    thresh(count) = eps;
    eps;
    
    % compute test statistic
    ef = 0;
    cc = 0;
    tstat = 0;
    % this loop just perturbs the initial guess a bit in case the optimizer gets stuck
    while ((ef ~= 1 & cc < 10) | (-100*tstat < eps) | (-tstat > 200*eps))
       delta = min(cc/1000,1);
       cc = cc + 1;
       alpha = 2*pi*rand;
       ind = ceil(4*rand);
       theta0 = theta(:,ind) + delta*[cos(alpha); sin(alpha)];
       [tstat, ef] = maxobj_rev(eps,nn,nn,taun,taun,n,n,supp,theta0);
    end
    if (ef ~= 1)
       ef
       eps
       -tstat
    end
    tstat = -n*tstat;

    % find the critical value for testing the hypothesis
    for i = 1:B
	    ef = 0;
	    cc = 0;
	    crit(i) = 0;
	    while ((ef ~= 1 & cc < 10) | (-100*crit(i) < eps) |( -crit(i) > 200*eps))
           delta = min(cc/1000,1);
	       cc = cc + 1;
           alpha = 2*pi*rand;
           ind = ceil(4*rand);
           theta0 = theta(:,ind) + delta*[cos(alpha); sin(alpha)];
           [crit(i),ef] = maxobj_rev(eps,nb(:,i),nn,taub(:,:,i),taun,b,n,supp,theta0);
        end
        if (ef ~= 1)
    	   ef
    	   eps
    	   -crit(i)
 	   	end
    end
    crit = -b*crit;
    crit = sort(crit);
    
    % accept or reject?
    if (tstat > crit(ceil(.95*B)))
       % check to see if any noticable improvement
       if (abs(eps - crit(ceil(.95*B))/n) < 10^(-4))
           flag = 1;
       else
           eps = crit(ceil(.95*B))/n;
       end    
    else
       flag = 1;
    end
end
sdthresh(m,c) = thresh(count);
if (count > 1)
   sd2thresh(m,c) = thresh(count - 1);
else
   sd2thresh(m,c) = thresh(count);
end

% compute critical values from single step procedure
% starting with .95 quantile of chi^2 with df = 2
eps = 5.991;  
for i = 1:B
    ef = 0;
    cc = 0;
    crit(i) = 0;
	while ((ef ~= 1 & cc < 10) | (-100*crit(i) < eps) | (-crit(i) > 200*eps))
       delta = min(cc/1000,1);
	   cc = cc + 1;
       alpha = 2*pi*rand;
       ind = ceil(4*rand);
       theta0 = theta(:,ind) + delta*[cos(alpha); sin(alpha)];
       [crit(i),ef] = maxobj_rev(eps,nb(:,i),nn,taub(:,:,i),taun,b,n,supp,theta0);
    end  
    if (ef ~= 1)
       ef
       eps
       -crit(i)
    end
end
crit = -b*crit;
crit = sort(crit);
ssthresh(m,c) = crit(ceil(.95*B))/n;

% check to see if iteration would have improved confidence region even starting
% with .95 quantile of chi^2 with df = 2

	% compute test statistic
	ef = 0;
	cc = 0;
	tstat = 0;
	while ((ef ~= 1 & cc < 10) | (-100*tstat < ssthresh(m,c)) | (-tstat > 200*ssthresh(m,c)))
        delta = min(cc/1000,1);
		cc = cc + 1;
		alpha = 2*pi*rand;
    	ind = ceil(4*rand);
    	theta0 = theta(:,ind) + delta*[cos(alpha); sin(alpha)];	
		[tstat, ef] = maxobj_rev(ssthresh(m,c),nn,nn,taun,taun,n,n,supp,theta0);
    end
    if (ef ~= 1)
       ef
       ssthresh(m,c)
       -tstat
    end
    tstat = -n*tstat;
    
    % find the critical value for testing the hypothesis
    for i = 1:B
        ef = 0;
        cc = 0;
        crit(i) = 0;
        while ((ef ~= 1 & cc < 10) | (-100*crit(i) < ssthresh(m,c)) | (-crit(i) > 200*ssthresh(m,c)))
	        delta = min(cc/1000,1);
			cc = cc + 1;
        	alpha = 2*pi*rand;
        	ind = ceil(4*rand);
        	theta0 = theta(:,ind) + delta*[cos(alpha); sin(alpha)];     
        	[crit(i), ef] = maxobj_rev(ssthresh(m,c),nb(:,i),nn,taub(:,:,i),taun,b,n,supp,theta0);
        end
	    if (ef ~= 1)
    	   ef
    	   ssthresh(m,c)
    	   -crit(i)
    	end
    end
    crit = -b*crit;
    crit = sort(crit);
        
    % check to see if iteration leads to improvement
    impr = (tstat > crit(ceil(.95*B)));

% check to see if identified set covered
objmax(m,c) = -1*min([obj(nn,taun,n,supp,theta(:,1)); obj(nn,taun,n,supp,theta(:,2)); obj(nn,taun,n,supp,theta(:,3)); obj(nn,taun,n,supp,theta(:,4))]);
coversd = (objmax(m,c) <= sdthresh(m,c));
coverss = (objmax(m,c) <= ssthresh(m,c));

% make a count of relevant things

	% fraction of times identified set covered
	percoversd(c,1) = percoversd(c,1) + coversd;
	percoverss(c,1) = percoverss(c,1) + coverss;

	% fraction of times iteration leads to improvement
	% even starting with .95 quantile of chi^2 with df = 2
	perimprss(c,1) = perimprss(c,1) + impr;

	% average number of iterations
	avgiter(c,1) = avgiter(c,1) + count;
	
	% fraction of times iterate more than once
	periter2(c,1) = periter2(c,1) + (count > 2);

% end loop over simulations
end

% end loop over subsample sizes
end

% compute averages
percoversd = percoversd/M;
percoverss = percoverss/M;
perimprss = perimprss/M;
avgiter = avgiter/M;
periter2 = periter2/M;

% compute quantiles of thresholds
ssthresh = sort(ssthresh);
sdthresh = sort(sdthresh);
sd2thresh = sort(sd2thresh);
objmax = sort(objmax);
i25 = ceil(.25*M);
i50 = ceil(.50*M);
i75 = ceil(.75*M);

table = [percoversd percoverss perimprss avgiter periter2 min(ssthresh)' min(sdthresh)' min(sd2thresh)' min(objmax)' ssthresh(i25,:)' sdthresh(i25,:)' sd2thresh(i25,:)' objmax(i25,:)' ssthresh(i50,:)' sdthresh(i50,:)' sd2thresh(i50,:)' objmax(i50,:)' ssthresh(i75,:)' sdthresh(i75,:)' sd2thresh(i75,:)' objmax(i75,:)' max(ssthresh)' max(sdthresh)' max(sd2thresh)' max(objmax)']